#include <QDir>
#include <QTextStream>
#include <QFile>
#include <QRegExp>
#include <QMap>

int main(int argc, char *argv[])
{
    // Creiamo un wrapper Qt per lo standard output (la console)
    QTextStream console(stdout);

    // Controlliamo che ci sia stata passata l'espressione regolare
    if (argc != 2) {
        console << "Numero di argomenti errato" << endl;
        return -1;
    }

    // Array associativo che conterrà il nome del file e occorrenze
    // dell'espressione regolare specificata
    QMap<QString,int> results;

    // Inizializziamo la regexp al primo argomento ricevuto
    QRegExp regexp(argv[1]);

    // Prendiamo i file nella cartella corrente
    QDir dir;
    QList<QString> files = dir.entryList(QDir::Files);

    // Cicliamo su tutti i nomi di file
    foreach (QString fileName, files)
    {
        // Apriamo il file in lettura
        QFile file(fileName);
        file.open(QFile::ReadOnly);

        // Creiamo uno stream testuale
        QTextStream reader(&file);
        // Leggiamo per intero il file
        QString contents = reader.readAll();

        // Chiudiamo il file
        file.close();

        // Posizione dove inizia la corrispondenza con l'espression regolare
        int position = 0;
        // Conteggio delle occorrenze
        int occurrences = 0;

        // Finché ci sono nuove occorrenze, contale
        // QRegExp::indexIn restituisce -1 se non vi è corrispondenza
        position = regexp.indexIn(contents);
        while (position != -1)
        {
            occurrences++;
            position = regexp.indexIn(contents, position + 1);
        }

        // Se c'è almeno un'occorrenza aggiungiamo il file alla
        // QMap dei risultati
        if (occurrences > 0)
            results.insert(fileName, occurrences);

    }

    // Stampiamo i risultati
    // Cicliamo sulle chiavi della QMap
    foreach (QString fileName, results.keys())
    {
        // Usiamo QString::arg per costruire la stringa
        // results[fileName] restituisce il valore corrispondente alla
        // chiave fileName
        console << QString("%1 ha %2 occorrenze")
                   .arg(fileName)
                   .arg(results[fileName])
                << endl;
    }

    return 0;
}
